<?php
/**
 * @package Joomla
 * @subpackage Abook
 * @copyright (C) 2010-2019 Federica Ugolotti
 * @license GNU/GPL, see LICENSE.php
 * Abook is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 3
 * as published by the Free Software Foundation.

 * Abook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Abook; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

defined('JPATH_BASE') or die;

jimport('joomla.html.html');
jimport('joomla.form.formfield');
jimport('joomla.form.helper');
JFormHelper::loadFieldClass('list');
/**
 * Form Field class for the Joomla Framework.
 *
 * @package		Joomla.Administrator
 * @subpackage	com_categories
 * @since		1.6
 */
//select per la scelta della parent category nel form di inserimento delle categorie del backend
class JFormFieldCategoryParent extends JFormFieldList
{
	/**
	 * The form field type.
	 *
	 * @var		string
	 * @since	1.6
	 */
	protected $type = 'CategoryParent';
	/**
	 * Method to get the field options.
	 *
	 * @return	array	The field option objects.
	 * @since	1.6
	 */
	protected function getOptions()
	{
		// Initialize variables.
		$options = array();
		$name = (string) $this->element['name'];

                // Let's get the id for the current item, either category or content item.
                $jinput = JFactory::getApplication()->input;

                // For categories the old category is the category id 0 for new category.
                if ($this->element['parent'])
                {
                        $oldCat = $jinput->get('id', 0);
                }
                else
                        // For items the old category is the category they are in when opened or 0 if new.
                {
                        $oldCat = $this->form->getValue($name);
                }

		$db = JFactory::getDbo();
		$query	= $db->getQuery(true);

		$query->select('a.id AS value, a.title AS text, a.level');
		$query->from('#__abcategories AS a');
		$query->join('LEFT', '#__abcategories AS b ON a.lft > b.lft AND a.rgt < b.rgt');

		if ($this->element['parent'])
                {
		// Prevent parenting to children of this item.
		if ($id = $this->form->getValue('id')) {
			$query->join('LEFT', '#__abcategories AS p ON p.id = '.(int) $id);
			$query->where('NOT(a.lft >= p.lft AND a.rgt <= p.rgt)');
			$rowQuery = $db->getQuery(true);
                                $rowQuery->select('a.id AS value, a.title AS text, a.level, a.parent_id')
                                        ->from('#__categories AS a')
                                        ->where('a.id = ' . (int) $id);
                                $db->setQuery($rowQuery);
                                $row = $db->loadObject();
		}
		}
		$query->where('a.published IN (0,1)');
		$query->group('a.id');
		$query->order('a.lft ASC');

		try{
		    $db->setQuery($query);
		    $options = $db->loadObjectList();
		}
		catch (RuntimeException $e){
		    JFactory::getApplication()->enqueueMessage($db->getErrorMsg(), 'error');
		}

		// Pad the option text with spaces using depth level as a multiplier.
		for ($i = 0, $n = count($options); $i < $n; $i++) {
			// Translate ROOT
			if ($options[$i]->level == 0) {
				$options[$i]->text = JText::_('JGLOBAL_ROOT_PARENT');
				$options[$i]->value = 0;
			}
			$options[$i]->text = str_repeat('- ',$options[$i]->level).$options[$i]->text;
		}

		// Merge any additional options in the XML definition.
		$options = array_merge(parent::getOptions(), $options);

		return $options;
	}
}
